/*
 *
 *  Copyright (c) 2002-2005 by Danilo Krahn
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, Or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY Or FITNESS For A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License For more details. 
 *
 */
#include "SkinWinFast.h"
#include <stdio.h>



long file_error = 0;


void WriteLong(FILE *f, unsigned long value) {
   if(!file_error)
      if(fwrite(&value,sizeof(unsigned long),1,f)!=1) file_error=1;
}



PBFUNCTION(long) PB_GenerateFastSkin2(char *file_skin,
                                      char *file_skin_image,
                                      char *file_background,
                                      unsigned long compress,
                                      long          x,
                                      long          y) {
   FILE             *f;
   size_t            size;
   BITMAPFILEHEADER *bfh;
   BITMAPINFO       *bi;
   HBITMAP           skin_image=0;

   unsigned long     skin_width = 0;
   unsigned long     skin_height = 0;
   
   HRGN              skin_region;
   unsigned long     skin_region_size;
   RGNDATA          *skin_region_mem=0;
   
   void             *background_mem =0;
   unsigned long     background_size=0;
   
   void             *finalmem;
   unsigned long     finalmem_size;
   
   void             *compressed_mem;
   unsigned long     compressed_size;
   

   if(file_skin && file_skin_image) {



      //****************
      //
      // open skin image
      //
      //****************
      if(f = fopen(file_skin_image,"rb")) {
         // go to end of file
         if(!fseek(f,0,SEEK_END)) {
            // get current position (size of file)
            if(size = ftell(f)) {
              // check filesize for bitmaps
              if(size>(sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFO))) {
                  // go back to start
                  rewind(f);
  
                  // alloc memory
                  if(bfh = PB_AllocateMemory(size)) {
                     // read file into memory
                     fread((void*)bfh,size,1,f);

                     // catch bitmap from memory
                     HDC DesktopDC  = GetDC(GetDesktopWindow());
                     if(DesktopDC) {
                        bi = (BITMAPINFO*)((unsigned long)bfh + sizeof(BITMAPFILEHEADER));
                        skin_width  = bi->bmiHeader.biWidth;
                        skin_height = bi->bmiHeader.biHeight;
                        if(skin_image = CreateCompatibleBitmap(DesktopDC,
                                                               skin_width,
                                                               skin_height)) {
                           HDC hDC = CreateCompatibleDC(0);
                           if(!SetDIBits(hDC,skin_image,0,skin_height,
                                         (void*)((unsigned long)bfh + bfh->bfOffBits),
                                         bi,DIB_RGB_COLORS)) {
                              DeleteObject(skin_image);
                              skin_image = 0;
                           }
                           DeleteDC(hDC);
                        }
                        ReleaseDC(GetDesktopWindow(),DesktopDC);
                     }

                     // free memory
                     PB_FreeMemory(bfh);
                  }
               }
            }
         }
         // close file
         fclose(f);
      }
      
      // failed to load and create the skin_image
      if(!skin_image) return 0;



      //****************
      //
      // make region
      //
      //****************
      if(!(skin_region = PB_SkinWinMakeRegion(skin_image,x,y))) {
         // failed
         DeleteObject(skin_image);
         return 0;
      }
      
      // free skin image, not needed anymore
      DeleteObject(skin_image);



      //****************
      //
      // get region data
      //
      //****************
      if(!(skin_region_size = GetRegionData(skin_region,0,0)))      return 0;
      if(!(skin_region_mem  = PB_AllocateMemory(skin_region_size))) return 0;
      if(!GetRegionData(skin_region,skin_region_size,skin_region_mem)) {
         DeleteObject(skin_region);
         PB_FreeMemory(skin_region_mem);
         return 0;
      }

      // free skin region, not needed anymore
      DeleteObject(skin_region);



      //****************
      //
      // load background, if any
      //
      //****************

      // if file_background == 1, use the skin file as background
      if((long)file_background == 1) file_background = file_skin_image;

      // load background
      if(file_background) {
         // open background image
         if(f = fopen(file_background,"rb")) {
            // go to end of file
            if(!fseek(f,0,SEEK_END)) {
               // get current position (size of file)
               if(background_size = ftell(f)) {
                  // check filesize for bitmaps
                  if(background_size>(sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFO))) {
                     // go back to start
                     rewind(f);

                     // alloc memory
                     if(background_mem = PB_AllocateMemory(background_size)) {
                        // read file into memory
                        if(fread((void*)background_mem,background_size,1,f)!=1) {
                           // failed
                           PB_FreeMemory(background_mem);
                           background_mem = 0;
                        }
                     }
                  }
               }
            }
            // close file
            fclose(f);
         }
         // couldnt load background
         if(!background_mem) {
            PB_FreeMemory(skin_region_mem);
            return 0;
         }
      }
      else background_size = 0;



      //****************
      //
      // copy region and background data to final memory
      //
      //****************
      finalmem_size = skin_region_size+background_size;
      if(!(finalmem = PB_AllocateMemory(finalmem_size))) {
         // failed
         PB_FreeMemory(skin_region_mem);
         if(background_mem) PB_FreeMemory(background_mem);
         return 0;
      }
      
      PB_CopyMemory(skin_region_mem,finalmem,skin_region_size);
      if(background_mem)
         PB_CopyMemory(background_mem,(void*)((unsigned long)finalmem + skin_region_size),background_size);

      // OK, free region and background memory
      PB_FreeMemory(skin_region_mem);
      if(background_mem) PB_FreeMemory(background_mem);



      //****************
      //
      // compress region and background
      //
      //****************
      if(compress) {
	     if(sw_LZMA_Initialized == 0) {
			PB_UseLZMAPacker();
			sw_LZMA_Initialized = 1;
		 }

         if(compressed_mem = PB_AllocateMemory(finalmem_size+8)) {
            //if(compressed_size = PB_PackMemory2(finalmem,compressed_mem,finalmem_size,9)) {
		    if(compressed_size = PB_CompressMemory2(finalmem,finalmem_size,compressed_mem,finalmem_size+8,PB_PackerPlugin_LZMA)) {
               PB_FreeMemory(finalmem);
               finalmem      = compressed_mem;
               finalmem_size = compressed_size;
            }
            else compress = 0;
         }
         else {
            // cant allocate memory for compression,
            // so store it uncompressed
            compress = 0;
         }
      }



      //****************
      //
      // write skin file
      //
      //****************
      if(!(f = fopen(file_skin,"wb"))) {
         // opening file for writing failed
         PB_FreeMemory(finalmem);
         return 0;
      }
      

      WriteLong(f, 0x53504B44);                       // 'DKPS'
      WriteLong(f, 1);                                // Version: 1
      WriteLong(f, sizeof(_sw_DKPS));                 // size of header
      WriteLong(f, compress);                         // compressed?
      WriteLong(f, skin_region_size+background_size); // size_of_uncompressed_data
      WriteLong(f, finalmem_size);                    // size_of_compressed_data
      WriteLong(f, skin_region_size);                 // size_of_uncompressed_region_data
      WriteLong(f, 0);                                // region offset
      WriteLong(f, background_size);                  // size_of_uncompressed_bitmap
      WriteLong(f, 0+skin_region_size);               // bitmap_offset
      WriteLong(f, skin_width);                       // skin_width
      WriteLong(f, skin_height);                      // skin_height
      
      if(!file_error)
         if(fwrite(finalmem,finalmem_size,1,f)!=1) file_error=1;

      fflush(f);
      fclose(f);

      PB_FreeMemory(finalmem);
      if(!file_error) return 1;
   }
   return 0;
}
